import re, time
import numpy as np

import zhinst.ziPython
import zhinst.utils

def run_example(port=None,amplitude=0.100):
    """
    Configure some settings on a Zurich Instruments device, obtain a single
    demodulator sample via getSample() and calculate its RMS amplitude.

    Additional configuration: Connect signal output 1 to signal input 1 with a
    BNC cable.
    
    Note: This is intended to be a simple example demonstrating how to connect
    to a Zurich Instruments device from ziPython. In most cases, data
    acquistion should use either ziDAQServer's poll() method or instance of
    the ziDAQRecorder class.
    """
    # open a connection to a Zurich Instruments server
    daq = zhinst.utils.autoConnect(port)
    # note: to specify directly which server you connect to please use the
    # following with port=8005 for HF2 and port=8004 for UHF:
    # daq = ziDAQServer('localhost',port)

    # detect device
    device = zhinst.utils.autoDetect(daq)

    # find out whether the device is an HF2 or a UHF
    devtype = daq.getByte('/%s/features/devtype' %device)
    options = daq.getByte('/%s/features/options' %device)

    # configure the device for the sweep
    out_c = 0
    in_c = 0
    demod_c = 0
    osc_c = 0
    demod_rate = 10e3
    if re.match('UHF',devtype) and (not re.match('MF',options)):
        out_mixer_c = 3
    elif re.match('HF2',devtype) and (not re.match('MF',options)):
        out_mixer_c = 6
    else:
        out_mixer_c = 0

    # create a base configuration: disable all outputs, demods and scopes
    general_setting = [
         ['/%s/demods/*/rate' %device, 0],
         ['/%s/demods/*/trigger' %device, 0],
         ['/%s/sigouts/*/enables/*' %device, 0]
    ]
    # if the device is a UHF additionally disable all demodulators
    if re.match('UHF',devtype):
        general_setting.append( ['/%s/demods/*/enable' %device, 0] )
        general_setting.append( ['/%s/scopes/*/enable' %device, 0] )
    elif re.match('HF2',devtype):
        general_setting.append( ['/%s/scopes/*/trigchannel' %device, -1 ])
    daq.set(general_setting)

    # now set the settings relevant to this experiment
    exp_setting = [ 
        ['/%s/sigins/%d/imp50' %(device,in_c), 0],
        ['/%s/sigins/%d/ac'    %(device,in_c), 0],
        ['/%s/sigins/%d/range' %(device,in_c), 2*amplitude],

        ['/%s/demods/%d/order'        %(device,demod_c), 4],
        ['/%s/demods/%d/timeconstant' %(device,demod_c), 1e-6],
        ['/%s/demods/%d/rate'         %(device,demod_c), demod_rate],
        ['/%s/demods/%d/harmonic'     %(device,demod_c), 1],
        
        ['/%s/oscs/%d/freq'        %(device,osc_c), 1e6],

        ['/%s/sigouts/%d/on'            %(device,out_c), 1],
        ['/%s/sigouts/%d/enables/%d'    %(device,out_c,out_mixer_c), 1],
        ['/%s/sigouts/%d/range'         %(device,out_c), 1],
        ['/%s/sigouts/%d/amplitudes/%d' %(device,out_c,out_mixer_c), amplitude]
    ]
    # device dependent configuration
    if re.match('UHF',devtype):
        exp_setting.append( ['/%s/demods/%d/enable' %(device,demod_c), 1] )
        if re.match('.*MF',options):
            exp_setting.append( ['/%s/demods/%d/oscselect' %(device,demod_c), osc_c] )
            exp_setting.append( ['/%s/demods/%d/adcselect' %(device,demod_c), in_c] )
    elif re.match('HF2',devtype):
        exp_setting.append( ['/%s/demods/%d/oscselect' %(device,demod_c), osc_c] )
        exp_setting.append( ['/%s/demods/%d/adcselect' %(device,demod_c), in_c] )
        exp_setting.append( ['/%s/sigins/%d/diff' %(device,in_c), 0] )
        exp_setting.append( ['/%s/sigouts/%d/add'  %(device,out_c), 0] )
    else:
        raise AssertionError, ("Bad device type: '%s'"%device)
    daq.set(exp_setting);

    # wait for the demodulator filter to settle
    time.sleep( 10*daq.getDouble('/%s/demods/0/timeconstant'%device) )

    # obtain one demodulator sample
    sample = daq.getSample('/%s/demods/0/sample' %device)
    R = np.sqrt(sample['x']**2+sample['y']**2)
    print 'Measured rms amplitude is %fV.' % R
    return R

if __name__ == '__main__':
    R = run_example()
