import re, time
import numpy as np

import zhinst.ziPython as ziPython
import zhinst.utils as utils

def run_example(port=None,amplitude=0.5,do_plot=False):
    """
    Save demodulator data using ziDAQServer's non-blocking (synchronous)
    poll() command.
    """
    # open a connection to a Zurich Instruments server
    daq = utils.autoConnect(port)
    # detect a device
    device = utils.autoDetect(daq)
    # find out whether the device is an HF2 or a UHF
    devtype = daq.getByte('/%s/features/devtype' %device)
    options = daq.getByte('/%s/features/options' %device)

    # define and configure the device
    frequency = 1e5
    in_c = 0
    out_c = 0
    demod_c = 0
    osc_c = 0
    amplitude = 0.500
    demod_rate = 200
    tc = 0.01
    if re.match('UHF',devtype) and (not re.match('MF',options)):
        out_mixer_c = 3
    elif re.match('HF2',devtype) and (not re.match('MF',options)):
        out_mixer_c = 6
    else:
        out_mixer_c = 0

    # create a base configuration: disable all outputs, demods and scopes
    general_setting = [
         ['/%s/demods/*/rate' %device, 0],
         ['/%s/demods/*/trigger' %device, 0],
         ['/%s/sigouts/*/enables/*' %device, 0]
    ]
    # if the device is a UHF additionally disable all demodulators
    if re.match('UHF',devtype):
        general_setting.append( ['/%s/demods/*/enable' %device, 0] )
        general_setting.append( ['/%s/scopes/*/enable' %device, 0] )
    elif re.match('HF2',devtype):
        general_setting.append( ['/%s/scopes/*/trigchannel' %device, -1 ])
    daq.set(general_setting)

    # now set the settings relevant to this experiment
    exp_setting = [ 
        ['/%s/sigins/%d/imp50' %(device,in_c), 1],
        ['/%s/sigins/%d/ac'    %(device,in_c), 0],
        ['/%s/sigins/%d/range' %(device,in_c), 2*amplitude],
        
        ['/%s/demods/%d/order'        %(device,demod_c), 4],
        ['/%s/demods/%d/timeconstant' %(device,demod_c), tc],
        ['/%s/demods/%d/rate'         %(device,demod_c), demod_rate],
        ['/%s/demods/%d/harmonic'     %(device,demod_c), 1],

        ['/%s/oscs/%d/freq'        %(device,osc_c), frequency],

        ['/%s/sigouts/%d/on'            %(device,out_c), 1],
        ['/%s/sigouts/%d/enables/%d'    %(device,out_c,out_mixer_c), 1],
        ['/%s/sigouts/%d/range'         %(device,out_c), 1],
        ['/%s/sigouts/%d/amplitudes/%d' %(device,out_c,out_mixer_c), amplitude]
    ]
    # device dependent configuration
    if re.match('UHF',devtype):
        exp_setting.append( ['/%s/demods/%d/enable' %(device,demod_c), 1], )
        if re.match('.*MF',options):
            exp_setting.append( ['/%s/demods/%d/oscselect' %(device,demod_c), osc_c] )
            exp_setting.append( ['/%s/demods/%d/adcselect' %(device,demod_c), in_c] )
    elif re.match('HF2',devtype):
        exp_setting.append( ['/%s/demods/%d/oscselect' %(device,demod_c), osc_c] )
        exp_setting.append( ['/%s/demods/%d/adcselect' %(device,demod_c), in_c] )
        exp_setting.append( ['/%s/sigins/%d/diff' %(device,in_c), 0] )
        exp_setting.append( ['/%s/sigouts/%d/add'  %(device,out_c), 0] )
    else:
        raise AssertionError, ("Bad device type: '%s'"%device)
    daq.set(exp_setting);

    # wait to get a settled lowpass filter
    time.sleep(1.0)
    # clean data buffers
    daq.flush()
    daq.unsubscribe('*')
    # subscribe to the demodulator's sample
    path = '/%s/demods/%d/sample' %(device,demod_c)
    daq.subscribe(path)

    # poll data for 1s
    poll_length = 1
    poll_timeout = 500
    data = daq.poll( poll_length, poll_timeout );
    # unsubscribe from all paths
    daq.unsubscribe('*')
  
    # check the dictionary returned is non-empty
    assert data, "poll returned an empty data dictionary, did you subscribe to any paths?"
    # note: data could be empty if no data arrived, e.g., if the demods were
    # disabled or had rate 0
    assert device in data, "data dictionary has no key '%s'" %(device)
    # the data returned is a dictionary of dictionaries that reflects the node's path
    sample = data[device]['demods'][str(demod_c)]['sample']
    R = np.sqrt(sample['x']**2 + sample['y']**2)
    print 'Measured average RMS amplitude is %.5fV.' %np.mean(R)

    if do_plot:
        import matplotlib
        import matplotlib.pyplot as plt

        # get the sample rate of the device's ADCs
        if re.match('HF2',devtype):
            # the sample rate of the HF2's ADCs
            clockbase = 210.0e6
        elif re.match('UHF',devtype):
            clockbase = daq.getInt('/%s/clockbase' %device)
            # ensure float to avoid problems with integer division
            clockbase = float(clockbase)
        R = np.sqrt(sample['x']**2 + sample['y']**2)
        expected_R = 0.5*amplitude/np.sqrt(2)
        t = (sample['timestamp']-sample['timestamp'][0])/clockbase

        # Create plot 
        plt.figure()
        plt.grid(True)
        plt.plot(t,R)
        plt.title('Demodulator data')
        plt.xlabel('Time (s)')
        plt.ylabel('R (V)')
        average_R = np.mean(R)
        plt.axis([t[0], t[-1], 0.99*average_R, 1.01*average_R])
        plt.show()

    return sample

if __name__ == '__main__':
    sample = run_example()
